@@ -153,6 +153,13 @@ has tag_end => (
 );
 sub _build_tag_end { ':>' }
 
+has comment_pattern => (
+    is      => 'ro',
+    isa     => 'RegexpRef',
+    builder => '_build_comment_pattern',
+);
+sub _build_comment_pattern { $COMMENT }
+
 has shortcut_table => (
     is      => 'ro',
     isa     => 'HashRef[Str]',
@@ -288,11 +295,18 @@ sub split :method {
     my $tag_start     = $parser->tag_start;
     my $tag_end       = $parser->tag_end;
 
-    my $lex_line_code = defined($line_start) && qr/\A ^ [ \t]* \Q$line_start\E ([^\n]* \n?) /xms;
+    my $lex_line_code = defined($line_start)
+        && qr/\A ^ [ \t]* \Q$line_start\E ([^\n]* \n?) /xms;
+
     my $lex_tag_start = qr/\A \Q$tag_start\E ($CHOMP_FLAGS?)/xms;
 
+    # 'text' is a something without newlines
+    # follwoing a newline, $tag_start, or end of the input
     my $lex_text = qr/\A ( [^\n]*? (?: \n | (?= \Q$tag_start\E ) | \z ) ) /xms;
 
+    my $lex_comment = $parser->comment_pattern;
+    my $lex_code    = qr/(?: $lex_comment | $CODE )/xms;
+
     my $in_tag = 0;
 
     while($_) {
@@ -301,7 +315,7 @@ sub split :method {
             my $pos;
             while( ($pos = index $_, $tag_end, $start) >= 0 ) {
                 my $code = substr $_, 0, $pos;
-                $code =~ s/$CODE//xmsog;
+                $code =~ s/$lex_code//xmsg;
                 if(length($code) == 0) {
                     last;
                 }
@@ -566,6 +580,9 @@ sub init_symbols {
 
     $parser->symbol('include')  ->set_std(\&std_include);
 
+    $parser->symbol('last')  ->set_std(\&std_last);
+    $parser->symbol('next')  ->set_std(\&std_next);
+
     # macros
 
     $parser->symbol('cascade')  ->set_std(\&std_cascade);
@@ -574,7 +591,7 @@ sub init_symbols {
     $parser->symbol('before')   ->set_std(\&std_proc);
     $parser->symbol('after')    ->set_std(\&std_proc);
     $parser->symbol('block')    ->set_std(\&std_macro_block);
-    $parser->symbol('super')    ->set_std(\&std_marker);
+    $parser->symbol('super')    ->set_std(\&std_super);
     $parser->symbol('override') ->set_std(\&std_override);
 
     $parser->symbol('->')       ->set_nud(\&nud_lambda);
@@ -633,18 +650,19 @@ sub tokenize {
 
     local *_ = \$parser->{input};
 
+    my $comment_rx = $parser->comment_pattern;
+    my $id_rx      = $parser->identity_pattern;
     TRY: {
         my $i = 0;
         s{\G (\s) }{ $1 eq "\n" and ++$i; "" }xmsge;
         $parser->following_newline($i);
 
-        my $id_rx = $parser->identity_pattern;
-        if(s/\A ($id_rx)//xms){
-            return [ name => $1 ];
-        }
-        elsif(s/\A $COMMENT //xmso) {
+        if(s/\A $comment_rx //xmso) {
             redo TRY; # retry
         }
+        elsif(s/\A ($id_rx)//xms){
+            return [ name => $1 ];
+        }
         elsif(s/\A ($NUMBER | $STRING)//xmso){
             return [ literal => $1 ];
         }
@@ -1434,6 +1452,14 @@ sub std_for {
     my $proc = $symbol->clone(arity => 'for');
     $proc->first( $parser->expression(0) );
     $parser->pointy($proc, 1);
+    if($parser->token eq 'else') {
+        $parser->advance();
+        my $else = $parser->block();
+        $proc = $symbol->clone( arity => 'for_else',
+            first  => $proc,
+            second => $else,
+        )
+    }
     return $proc;
 }
 
@@ -1669,7 +1695,10 @@ sub std_print {
     if($parser->token->id ne ";") {
         $args = $parser->expression_list();
     }
-    my $stmt = $symbol->clone(first => $args, arity => 'print');
+    my $stmt = $symbol->clone(
+        arity => 'print',
+        first => $args,
+    );
     return $parser->finish_statement($stmt);
 }
 
@@ -1752,10 +1781,21 @@ sub std_cascade {
     return $parser->finish_statement($stmt);
 }
 
-# markers for the compiler
-sub std_marker {
+sub std_super {
     my($parser, $symbol) = @_;
-    my $stmt = $symbol->clone(arity => 'marker');
+    my $stmt = $symbol->clone(arity => 'super');
+    return $parser->finish_statement($stmt);
+}
+
+sub std_next {
+    my($parser, $symbol) = @_;
+    my $stmt = $symbol->clone(arity => 'loop_control', id => 'next');
+    return $parser->finish_statement($stmt);
+}
+
+sub std_last {
+    my($parser, $symbol) = @_;
+    my $stmt = $symbol->clone(arity => 'loop_control', id => 'last');
     return $parser->finish_statement($stmt);
 }
 
@@ -1900,7 +1940,9 @@ sub make_alias { # alas(from => to)
 
     my $stash = $parser->symbol_table;
     if(exists $parser->symbol_table->{$to}) {
-        Carp::croak("Cannot make an alias to an existing symbol ($from => $to)");
+        Carp::confess(
+            "Cannot make an alias to an existing symbol ($from => $to / "
+            . p($parser->symbol_table->{$to}) .")");
     }
 
     # make a snapshot
